import { ServerResponse } from "http";
import * as fs from "node:fs";
import { promisify } from "util";
import { exec } from "node:child_process";
import { createHash } from "node:crypto";
const mediaFolder = "media/";
const execPromise = promisify(exec);
const mediaDir = `${__dirname}/${mediaFolder}`;
if (!fs.existsSync(mediaDir)) fs.mkdirSync(mediaDir);
export default abstract class FileHandler {
  tmpFolder: string;
  html = "";
  hash: string;
  protected constructor(private _sourcePath: string) {
    const data = fs.readFileSync(_sourcePath);
    const md5sum = createHash("md5");
    md5sum.update(data);
    this.hash = md5sum.digest("hex");
    const folder = `${mediaFolder}${this.hash}/`;
    this.tmpFolder = `${__dirname}/${folder}`;
    this.sendString(
      `<script>window.history.pushState({},"", "${folder}index.html");</script>`,
    );
    if (!fs.existsSync(this.tmpFolder)) fs.mkdirSync(this.tmpFolder);
  }
  write(filename: string, data: string | Buffer, note: string) {
    fs.writeFileSync(this.tmpFolder + filename, data);
    this.sendString(
      `<a href="${filename}">${filename} (${note}) ${data.length} bytes</a>`,
    );
  }
  exists(filename: string) {
    return fs.existsSync(this.tmpFolder + filename);
  }
  read(filename: string) {
    return fs.readFileSync(this.tmpFolder + filename);
  }
  sendString(text: string | Buffer) {
    text += "<br>";
    this.html += text;
    this.actualSend(text);
  }
  async end(text?: string | Buffer) {
    if (text) this.sendString(text);
    if (!this.exists("index.html")) {
      this.write("index.html", this.html, "index.html");
      await execPromise("zip -r gezipt.zip *", { cwd: this.tmpFolder });
      this.write(
        "gezipt.zip",
        this.read("gezipt.zip"),
        "Alle gegevens in een zip",
      );
    }
    this.actualEnd();
  }
  protected abstract actualSend(text: string | Buffer): void;
  protected abstract actualEnd(): void;
}

export class ServerFileHandler extends FileHandler {
  constructor(
    private res: ServerResponse,
    sourcePath: string,
  ) {
    super(sourcePath);
  }
  actualSend(text: string | Buffer) {
    this.res.write(text);
  }
  actualEnd() {
    this.res.end();
  }
}

export class ConsoleFileHandler extends FileHandler {
  constructor(sourcePath: string) {
    super(sourcePath);
  }
  actualSend(text: string | Buffer) {
    console.log(text);
  }
  actualEnd() {
    console.log("end");
  }
}
