import { stripHtmlTags } from "./utils";
import FileHandler from "./FileHandler";

export function searchCoordinates(luaSource: string, fileHandler: FileHandler) {
  const matched: string[] = luaSource.match(
    /.*.Points = {(\n.*){1,10}}/gm,
  ) as string[];
  if (!matched) return fileHandler.sendString("geen coordinaten gevonden");
  const rawStartingLocation = luaSource.match(/.*StartingLocation = .*/gm)![0];
  const cacheName = stripHtmlTags(
    luaSource
      .match(
        new RegExp(rawStartingLocation.split(".")[0] + ".Name = .*", "gm"),
      )![0]
      .split('"')
      .slice(1, -1)
      .join('"'),
  );
  const author = stripHtmlTags(
    luaSource
      .match(
        new RegExp(rawStartingLocation.split(".")[0] + ".Author = .*", "gm"),
      )![0]
      .split('"')
      .slice(1, -1)
      .join('"'),
  );
  const rawCreateDate = luaSource
    .match(
      new RegExp(rawStartingLocation.split(".")[0] + ".CreateDate = .*", "gm"),
    )![0]
    .split('"')
    .slice(1, -1)
    .join('"')
    .split(" ");
  const date = rawCreateDate[0].split("/");
  const createDate = `${date[2]}-${date[1]}-${date[0]}T${rawCreateDate[1]}Z`;
  const startingLocation = rawStartingLocation
    .split("(")[1]
    .split(")")[0]
    .split(", ");
  let gpx = `<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><gpx version="1.0" creator="c:geo - http://www.cgeo.org/" xsi:schemaLocation="http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd http://www.groundspeak.com/cache/1/0/1 http://www.groundspeak.com/cache/1/0/1/cache.xsd http://www.gsak.net/xmlv1/6 http://www.gsak.net/xmlv1/6/gsak.xsd" xmlns="http://www.topografix.com/GPX/1/0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:groundspeak="http://www.groundspeak.com/cache/1/0/1" xmlns:gsak="http://www.gsak.net/xmlv1/6" xmlns:cgeo="http://www.cgeo.org/wptext/1/0"><wpt lat="${startingLocation[0]}" lon="${startingLocation[1]}"><time>${createDate}</time><name>${cacheName}</name><desc>${cacheName}</desc><url>https://example.com</url><urlname>${cacheName}</urlname><sym>Geocache</sym><type>Geocache|Wherigo Cache</type><groundspeak:cache id="${fileHandler.hash}" available="True" archived="False"><groundspeak:name>${cacheName}</groundspeak:name><groundspeak:placed_by>${author}</groundspeak:placed_by><groundspeak:owner id="${fileHandler.hash}">${author}</groundspeak:owner><groundspeak:type>Wherigo Cache</groundspeak:type></groundspeak:cache><gsak:wptExtension><gsak:Watch>false</gsak:Watch><gsak:IsPremium>false</gsak:IsPremium><gsak:FavPoints>0</gsak:FavPoints><gsak:GcNote></gsak:GcNote></gsak:wptExtension><cgeo:cacheExtension><cgeo:assignedEmoji>0</cgeo:assignedEmoji></cgeo:cacheExtension></wpt>`;
  const locations = matched.map(function (zone) {
    const zoneNaam = zone.split(".")[0];
    const name = stripHtmlTags(
      luaSource
        .match(new RegExp(zoneNaam + ".Name = .*", "gm"))![0]
        .split('"')
        .slice(1, -1)
        .join('"'),
    );
    const description = stripHtmlTags(
      luaSource
        .match(new RegExp(zoneNaam + ".Description = .*", "gm"))![0]
        .split('"')
        .slice(1, -1)
        .join('"'),
    );
    return {
      coordinates: zone
        .split("\n")
        .slice(1, -1)
        .filter((row) => row.includes("("))
        .map((row) => row.split("(")[1].split(")")[0]),
      name,
      description,
    };
  });
  locations.map((location, id) => {
    const points = location.coordinates;
    gpx += `<wpt lat="${points[0].split(", ")[0]}" lon="${
      points[0].split(", ")[1]
    }">
			<name>${
        String(id).padStart(2, "0") + cacheName.split("").slice(2).join("")
      }</name>
			<cmt>${location.description}</cmt>
			<desc>${location.name}</desc>
			<sym></sym>
			<type>Waypoint|Physical Stage</type>
			<gsak:wptExtension>
			<gsak:Parent>${cacheName}</gsak:Parent>
			</gsak:wptExtension>
		</wpt>`;
  });
  gpx += "</gpx>";
  fileHandler.write("coordinates.gpx", gpx, "gpx");
  const htmlLocations = locations.map(
    (location) =>
      location.name +
      ":<ul>" +
      location.coordinates.map((point) => `<li>${point}</li>`).join("\n") +
      "</ul>",
  );
  fileHandler.sendString(
    `<ul>${htmlLocations
      .map((location) => `<li>${location}</li>`)
      .join("\n")}</ul>`,
  );
}
