import * as fs from "node:fs";
import parseLuac from "./parseLuac";
import FileHandler from "./FileHandler";

export async function parseFile(fromfile: string, fileHandler: FileHandler) {
  try {
    if (fileHandler.exists("index.html")) {
      fileHandler.sendString("Eerder verwerkte gegevens van dit bestand:");
      return await fileHandler.end(fileHandler.read("index.html"));
    }
    fileHandler.sendString("Eerste keer dat dit bestand wordt verwerkt:");
    const gwcData = fs.readFileSync(fromfile);
    fileHandler.write("uploaded.gwc", gwcData, "uploaded.gwc");
    if (gwcData.slice(2, 6).toString() != "CART")
      throw new Error("not a whereigo");
    const fileCount = gwcData.readUInt16LE(7);
    const files = [];
    for (let i = 0; i < fileCount; i++) {
      files.push({ start: gwcData.readInt32LE(11 + i * 6) });
    }
    for (const file of files) {
      const fileId = files.indexOf(file);
      if (fileId == 0) {
        await parseLuac(gwcData, file.start, fileHandler);
        continue;
      }
      const valid = gwcData[file.start];
      if (!valid) continue;
      const fileType = gwcData.readInt32LE(file.start + 1);
      const fileLength = gwcData.readInt32LE(file.start + 5);
      const fileExtension =
        {
          1: "bmp",
          2: "png",
          3: "jpg",
          4: "gif",
          17: "wav",
          18: "mp3",
          19: "fdl",
          20: "sdn",
          21: "ogg",
          33: "swf",
          49: "txt",
        }[fileType] || "UNKNOWN";
      const fileName = `${fileId}.${fileExtension}`;
      const fileData = gwcData.slice(
        file.start + 9,
        file.start + 9 + fileLength,
      );
      fileHandler.write(fileName, fileData, `type ${fileType}`);
      let tag = "";
      if (fileType <= 4) tag = `<img src="${fileName}">`;
      if ([17, 18].includes(fileType))
        tag = `<audio controls><source src="${fileName}"></audio>`;
      if (fileType == 21) tag = `<video><source src="${fileName}"></video>`;
      if (tag) fileHandler.sendString(tag);
    }
    await fileHandler.end();
  } catch (e) {
    console.log(e);
    //@ts-expect-error e.message is a string, but typescript doesn't know that
    await fileHandler.end(e.message as string);
  }
}
