import { ServerFileHandler } from "./FileHandler";

process.chdir(__dirname);
import * as http from "node:http";
import * as fs from "node:fs";
import { downloadWhereigo } from "./downloadWhereigo";
import { downloadGeocaching } from "./downloadGeocaching";
import formidable from "formidable";
import { parseFile } from "./parseFile";

const index =
  '<form action="/" method="post" enctype="multipart/form-data"><input type="file" name="filetoupload"><br>OF<input type="text" name="url" placeholder="wherigo url"><br>OF<input type="text" name="gccode" placeholder="GC code"><br><input type="submit" value="start"></form>';
http
  .createServer(async (req, res) => {
    if (req.method == "GET") {
      if (req.url == "/") {
        res.writeHead(200, {
          "Content-Type": "text/html",
          "Content-Length": index.length,
        });
        res.end(index);
      } else {
        const filetype = req.url!.split(".").pop();
        const headers: { [key: string]: string } = {
          "Content-Type": "text/plain",
        };
        if (filetype == "html") headers["Content-Type"] = "text/html";
        if (filetype == "gpx") {
          headers["Content-Type"] = "application/gpx";
          headers["Content-Disposition"] =
            `attachment; filename="waypoints.gpx"`;
        }
        try {
          const data = fs.readFileSync(req.url!.substring(1));
          headers["Content-Length"] = "" + data.length;
          res.writeHead(200, headers);
          res.end(data);
        } catch {
          res.writeHead(404, { "Content-Type": "text/plain" });
          res.end("file not found.");
        }
      }
    }
    if (req.method == "POST") {
      const form = formidable();
      const [fields, files] = await form.parse(req);
      let file: string = files.filetoupload
        ? files.filetoupload[0].filepath
        : "";
      let whereigoId = fields.url![0];
      if (fields.gccode![0])
        whereigoId = await downloadGeocaching(fields.gccode![0]);
      if (whereigoId.length > 0) file = await downloadWhereigo(whereigoId);
      if (file.length == 0) return res.end("geen gegevens ontvangen?");
      res.writeHead(200, { "Content-Type": "text/html" });
      const fileHandler = new ServerFileHandler(res, file);
      await parseFile(file, fileHandler);
    }
  })
  .listen(8080);
