import { exec, spawn } from "node:child_process";
import FileHandler from "./FileHandler";
import { promisify } from "util";
const execPromise = promisify(exec);
export default async function handleUrwigo(
  luaSource: string,
  fileHandler: FileHandler,
) {
  let decodeFunction: string = "",
    searchFor: string,
    urwigo = false;
  if (luaSource.split("\n")[4].match(/function _.{3,5}\(str\)/)) {
    decodeFunction =
      "function decode(str)\n" + luaSource.split("\n").slice(5, 17).join("\n");
    searchFor =
      luaSource.split("\n")[4].match(/(?<=function )_\w*/g)![0] + '("';
    urwigo = true;
  } else if (luaSource.split("\n")[5].match(/function _.{3,5}\(str\)/)) {
    decodeFunction =
      "function decode(str)\n" + luaSource.split("\n").slice(6, 18).join("\n");
    searchFor =
      luaSource.split("\n")[5].match(/(?<=function )_\w*/g)![0] + '("';
    urwigo = true;
  }
  if (!urwigo) {
    fileHandler.sendString("geen urwigo");
    return luaSource;
  }
  fileHandler.sendString("Detected urwigo");
  const filtered = luaSource
    .split("\n")
    .filter(function (str) {
      return str.includes(searchFor);
    })
    .map(function (str) {
      return str.trim();
    })
    .map(function (line) {
      return line.split(searchFor);
    });
  const uitzonderingen = filtered.filter(function (line) {
    return line.length !== 2;
  });
  const uitzonderingenLijst: string[] = [];
  uitzonderingen.forEach(function (line) {
    line
      .filter(function (part, partId) {
        if (partId % 2 == 1) return true;
      })
      .forEach(function (part) {
        uitzonderingenLijst.push(
          part.substring(0, part.search(/[^\\]"\)/) + 1),
        ); //TODO lastIndexOf
      });
  });
  const normale = filtered
    .filter(function (line) {
      return line.length == 2;
    })
    .map(function (line) {
      return line[1].substring(0, line[1].search(/[^\\]"\)/) + 1);
    }); //TODO lastIndexOf
  normale.shift();
  const joined = normale.concat(uitzonderingenLijst);
  if (!decodeFunction) throw new Error("decodeFunction not found");
  const luacode = `${decodeFunction}
print(table.concat({decode("${joined.join('"),"$$$$~~~$$$$",decode("')}")}))`;
  const decodechild = spawn("/usr/bin/lua");
  decodechild.stdin.write(luacode);
  decodechild.stdin.end();
  let decoded = "";
  let decodestderr = "";
  decodechild.stdout.on("data", function (chunk) {
    decoded += chunk;
  });
  decodechild.stderr.on("data", function (chunk) {
    decodestderr += chunk;
  });
  await new Promise((resolve) => decodechild.on("close", resolve));
  if (decodestderr) {
    fileHandler.write(
      "urwigo.lua",
      decodestderr + "\n\nmet code:\n\n" + luacode,
      "urwigo DECOMPILE FAILED!",
    );
    return luaSource;
  }
  const replaceTo = decoded.split("$$$$~~~$$$$");
  joined.map(function (van, simpelId) {
    luaSource = luaSource.replace(
      searchFor + van + '")',
      '"' +
        replaceTo[simpelId].split('"').join('\\"').replace("\n", "\\n") +
        '"',
    );
  });
  const matches = luaSource.match(/(_.{3,5})\.Name = (".*")/g);
  if (matches) {
    matches.forEach(function (line) {
      const splitted = line.split('.Name = "');
      const typeRegex = luaSource.match(
        new RegExp(splitted[0] + " = .*\\(", "g"),
      );
      if (!typeRegex) return;
      const type = typeRegex[0].split(".")[1].slice(0, -1);
      luaSource = luaSource.replace(
        new RegExp(splitted[0], "g"),
        (type + splitted[1] + type)
          .replace(/[ #[\]=/&:,.\-()+!?"\\';]/g, "_")
          .replace(/[À-ÿ]/g, "_"),
      );
    });
  }
  fileHandler.write("urwigo.lua", luaSource, "urwigo de-obfuscated");
  try {
    await execPromise(
      `lua -e "assert(loadfile('${fileHandler.tmpFolder}urwigo.lua'))"`,
    );
    fileHandler.sendString("Deobfuscation is successful.");
  } catch (e) {
    fileHandler.sendString(
      "Deobfuscation failed, but the code should be more readable.",
    );
  }
  const MultipleChoicesearch = luaSource.match(
    /.*_ZInput.Choices = {(\n.*){1,10}}/gm,
  );
  if (MultipleChoicesearch) {
    const vragen: string[] = [];
    const antwoorden: string[][] = [];
    const vraagTexten: string[] = [];
    MultipleChoicesearch.forEach(function (vraag) {
      const vraagId = vraag.split(".")[0];
      vragen.push(vraagId);
      let vraagText = luaSource.match(
        new RegExp(`${vraagId}.Text = ".*$`, "gm"),
      );
      if (!vraagText)
        vraagText = luaSource.match(
          new RegExp(`${vraagId}.Name = ".*$`, "gm"),
        )!;
      vraagTexten.push(vraagText[0].split('"').slice(1, -1).join('"'));
      antwoorden.push(
        vraag
          .split("\n")
          .slice(1, -1)
          .map(function (antwoord) {
            if (antwoord.split('"')[1])
              return antwoord.split('"')[1].toLowerCase();
            else return "antwoord niet gevonden";
          }),
      );
    });
    const hashes = JSON.parse(
      JSON.stringify(new Array(vragen.length).fill([])),
    );
    const hashFunction =
      "function Hash(str)\n\tlocal b = 378551\n\tlocal a = 63689\n\tlocal hash = 0\n\tfor i = 1, #str do\n\t\thash = hash * a + string.byte(str, i)\n\t\thash = math.fmod(hash, 65535)\n\t\ta = a * b\n\t\ta = math.fmod(a, 65535)\n\tend\n\treturn hash\nend";
    const sendToLua = `${hashFunction}
print(table.concat({Hash("${antwoorden.join('"),"$$$$~~~$$$$",Hash("')}")}))`;
    const hashchild = spawn("/usr/bin/lua");
    hashchild.stdin.write(sendToLua);
    hashchild.stdin.end();
    let hashed = "";
    hashchild.stdout.on("data", function (chunk) {
      hashed += chunk;
    });
    hashchild.on("close", function () {
      let i = 0;
      hashed.split("$$$$~~~$$$$").forEach(function (hash) {
        if (antwoorden[i].length == hashes[i].length) i++;
        hashes[i].push(parseInt(hash));
      });
      const correcteHashes: number[] = [];
      vragen.forEach(function (vraag) {
        const regel = luaSource.match(
          new RegExp(
            vraag +
              ':OnGetInput\\(input\\)\n  if input == nil then\n    input = ""\n  end\n  if _Urwigo.Hash\\(string.lower\\(input\\)\\) == \\d*',
            "gm",
          ),
        );
        if (regel) correcteHashes.push(parseInt(regel[0].split(" == ")[2]));
        else correcteHashes.push(0);
      });
      fileHandler.sendString("De antwoorden op de MultipleChoice vragen:");
      correcteHashes.forEach(function (correcteHash, hashId) {
        fileHandler.sendString(
          `${vraagTexten[hashId]}=${
            antwoorden[hashId][hashes[hashId].indexOf(correcteHash)]
          }`,
        );
      });
    });
  } else {
    fileHandler.sendString("geen multiplechoice vragen.");
  }
  return luaSource;
}
