import { exec } from "node:child_process";
import FileHandler from "./FileHandler";
import { promisify } from "util";
import { searchCoordinates } from "./coordinates";
import parseMultipleChoice from "./multipleChoice";
import handleUrwigo from "./urwigo";
import { gsub_wig, ucode_wig, WWB_deobf } from "./otherDeobfuscate";
const execPromise = promisify(exec);
export default async function parseLuac(
  gwcData: Buffer,
  start: number,
  fileHandler: FileHandler,
) {
  const length = gwcData.readInt32LE(start);
  const fileData = gwcData.slice(start + 4, start + 4 + length);
  fileHandler.write("compiled.luac", fileData, `luac decompiled(not readable)`);
  // eslint-disable-next-line prefer-const
  let { stdout: luaSource, stderr } = await execPromise(
    `/usr/bin/java -jar ${__dirname}/unluac.jar ${fileHandler.tmpFolder}compiled.luac`,
    { maxBuffer: 1024 * 1024 },
  );
  if (stderr) throw new Error("unluac errored:" + stderr);
  fileHandler.write("original.lua", luaSource, "raw lua source");
  luaSource = await handleUrwigo(luaSource, fileHandler);
  if (luaSource.match(/function gsub_wig\(str, obf\)/g)) {
    if (luaSource.match(/gsub_wig\(".*?"\)/g)) {
      luaSource
        .match(/gsub_wig\(".*?"\)/g)!
        .slice(0, -1)
        .forEach(function (match) {
          luaSource = luaSource.replace(
            match,
            `"${gsub_wig(match.split('"')[1])}"`,
          );
        });
    }
    if (luaSource.match(/ucode_wig\(.*?\)/g)) {
      luaSource
        .match(/ucode_wig\(.*?\)/g)!
        .slice(0, -2)
        .forEach(function (match) {
          luaSource = luaSource.replace(
            match,
            `"${ucode_wig(+match.split("(")[1].split(")")[0])}"`,
          );
        });
    }
    luaSource = luaSource.replace(/ \.\. gsub_wig\(""\)/g, "");
    luaSource = luaSource.replace(/ \.\. ""/g, "");
    fileHandler.write(
      "gsub_wig.lua",
      luaSource,
      "gsub_wig decompiled(readable)",
    );
  }
  if (luaSource.match(/WWB_deobf/g)) {
    luaSource.match(/WWB_deobf\(".*?"\)/g)!.forEach(function (match) {
      luaSource = luaSource.replace(
        match,
        '"' + WWB_deobf(match.split('"')[1]) + '"',
      );
    });
    fileHandler.write(
      "WWB_deobf.lua",
      luaSource,
      "WWB_deobf decompiled(readable)",
    );
  }
  //if (urwigo) return;
  parseMultipleChoice(luaSource, fileHandler);
  searchCoordinates(luaSource, fileHandler);
}
